package sortie.data.funcgroups.light;

import java.io.File;
import java.io.IOException;
import java.io.FileWriter;

import sortie.ModelTestCase;
import sortie.data.funcgroups.LightBehaviors;
import sortie.data.funcgroups.TreePopulation;
import sortie.data.simpletypes.ModelException;
import sortie.gui.GUIManager;

public class TestAverageLight extends ModelTestCase {
  
  public void testBackwardsCompatibility() {
    GUIManager oManager = null;
    String sFileName = null;
    String sNewFileName = "c:\\test7.xml";
    LightBehaviors oLightBeh = null;
    TreePopulation oPop = null;

    try {

      oManager = new GUIManager(null);
      oManager.clearCurrentData();
      sFileName = writeV6XMLFile();
      oManager.convertPre7ParameterFile(sFileName, sNewFileName);
      oManager.inputXMLParameterFile(sNewFileName);
      oLightBeh = oManager.getLightBehaviors();
      oPop = oManager.getTreePopulation();
      oLightBeh.validateData(oPop);
      
      
    } catch (ModelException oE) {
      fail("Caught ModelException.  Message: " + oE.getMessage());
    }
    catch (IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    } finally {
      new File(sNewFileName).delete();
      new File(sFileName).delete();
    }
  }

  /**
   * Tests validate data.
   */
  public void testValidateData() {
    GUIManager oManager = null;
    String sFileName = null;
    LightBehaviors oLightBeh = null;
    TreePopulation oPop = null;

    //Case:  Average light enabled but not GLI Map.
    try {

      oManager = new GUIManager(null);
      oManager.clearCurrentData();
      sFileName = writeXMLErrorFile1();
      oManager.inputXMLParameterFile(sFileName);
      oLightBeh = oManager.getLightBehaviors();
      oPop = oManager.getTreePopulation();
      oLightBeh.validateData(oPop);
      fail("Didn't catch lack of GLI Map with Average Light.");
    } catch (ModelException oErr) {
      ;
    }
    catch (IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Case:  GLI Map not before Average light.
    try {

      oManager = new GUIManager(null);
      oManager.clearCurrentData();
      sFileName = writeXMLErrorFile2();
      oManager.inputXMLParameterFile(sFileName);
      oLightBeh = oManager.getLightBehaviors();
      oPop = oManager.getTreePopulation();
      oLightBeh.validateData(oPop);
      fail("Didn't catch GLI Map after Average Light.");
    } catch (ModelException oErr) {
      ;
    }
    catch (IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a file with average light but no GLI Map behavior.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  private String writeXMLErrorFile1() throws IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"07010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLILight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>1</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>QuadratLight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>2</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>AverageLight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>3</listPosition>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<GeneralLight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</GeneralLight>");
    oOut.write("<GLILight1>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</GLILight1>");
    oOut.write("<QuadratLight2>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</QuadratLight2>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with the GLI map behavior after the average light behavior.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  private String writeXMLErrorFile2() throws IOException {
    String sFileName = "\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"07010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLILight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>1</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>QuadratLight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>2</listPosition>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>AverageLight</behaviorName>");
    oOut.write("<listPosition>3</listPosition>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLIMapCreator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<listPosition>4</listPosition>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<GeneralLight>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</GeneralLight>");
    oOut.write("<GLILight1>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("</GLILight1>");
    oOut.write("<QuadratLight2>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</QuadratLight2>");
    oOut.write("<GLIMapCreator4>");
    oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>14</li_numAltGrids>");
    oOut.write("<li_numAziGrids>20</li_numAziGrids>");
    oOut.write("<li_mapLightHeight>2.5</li_mapLightHeight>");
    oOut.write("</GLIMapCreator4>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file from version 6.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  private String writeV6XMLFile() throws IOException {
    String sFileName = "c:\\testfile1.xml";
    FileWriter oOut = new FileWriter(sFileName);
    
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06060101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>3</timesteps>");
    oOut.write("<randomSeed>0</randomSeed>");
    oOut.write("<yearsPerTimestep>5.0</yearsPerTimestep>");
    oOut.write("<plot_lenX>100.0</plot_lenX>");
    oOut.write("<plot_lenY>100.0</plot_lenY>");
    oOut.write("<plot_latitude>41.92</plot_latitude>");
    oOut.write("<plot_precip_mm_yr>1150.645781</plot_precip_mm_yr>");
    oOut.write("<plot_temp_C>12.88171785</plot_temp_C>");
    oOut.write("<plot_title>Default parameter file-use for testing only</plot_title>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"ACRU\"/>");
    oOut.write("<tr_species speciesName=\"ACSA\"/>");
    oOut.write("<tr_species speciesName=\"BEAL\"/>");
    oOut.write("<tr_species speciesName=\"FAGR\"/>");
    oOut.write("<tr_species speciesName=\"TSCA\"/>");
    oOut.write("<tr_species speciesName=\"FRAM\"/>");
    oOut.write("<tr_species speciesName=\"PIST\"/>");
    oOut.write("<tr_species speciesName=\"PRSE\"/>");
    oOut.write("<tr_species speciesName=\"QURU\"/>");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.2</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"ACRU\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"ACSA\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"BEAL\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"FAGR\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"TSCA\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"FRAM\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"PIST\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"PRSE\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"QURU\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("<tr_maxSeedlingHeight>");
    oOut.write("<tr_mshVal species=\"ACRU\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"ACSA\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"BEAL\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"FAGR\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"TSCA\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"FRAM\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"PIST\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"PRSE\">1.35</tr_mshVal>");
    oOut.write("<tr_mshVal species=\"QURU\">1.35</tr_mshVal>");
    oOut.write("</tr_maxSeedlingHeight>");
oOut.write("</trees>");
    oOut.write("<grid gridName=\"Average Light\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_lengthXCells>100</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>100</ma_lengthYCells>");
    oOut.write("</grid>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Seedling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Average Light</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>relunlimgrowth</behaviorName>");
    oOut.write("<version>1.1</version>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Seedling\"/>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Sapling\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>constradialgrowth</behaviorName>");
    oOut.write("<version>1.0</version>");
    oOut.write("<applyTo species=\"ACRU\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"ACSA\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"BEAL\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"FAGR\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"TSCA\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"FRAM\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"PIST\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"PRSE\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"QURU\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<allometry>");
    oOut.write("<tr_whatAdultHeightDiam>");
    oOut.write("<tr_wahdVal species=\"ACRU\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"ACSA\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"BEAL\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"FAGR\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"TSCA\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"FRAM\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"PIST\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"PRSE\">0</tr_wahdVal>");
    oOut.write("<tr_wahdVal species=\"QURU\">0</tr_wahdVal>");
    oOut.write("</tr_whatAdultHeightDiam>");
    oOut.write("<tr_whatSaplingHeightDiam>");
    oOut.write("<tr_wsahdVal species=\"ACRU\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"ACSA\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"BEAL\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"FAGR\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"TSCA\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"FRAM\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"PIST\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"PRSE\">0</tr_wsahdVal>");
    oOut.write("<tr_wsahdVal species=\"QURU\">0</tr_wsahdVal>");
    oOut.write("</tr_whatSaplingHeightDiam>");
    oOut.write("<tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_wsehdVal species=\"ACRU\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"ACSA\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"BEAL\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"FAGR\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"TSCA\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"FRAM\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"PIST\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"PRSE\">0</tr_wsehdVal>");
    oOut.write("<tr_wsehdVal species=\"QURU\">0</tr_wsehdVal>");
    oOut.write("</tr_whatSeedlingHeightDiam>");
    oOut.write("<tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_wacrdVal species=\"ACRU\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"ACSA\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"BEAL\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"FAGR\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"TSCA\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"FRAM\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"PIST\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"PRSE\">0</tr_wacrdVal>");
    oOut.write("<tr_wacrdVal species=\"QURU\">0</tr_wacrdVal>");
    oOut.write("</tr_whatAdultCrownRadDiam>");
    oOut.write("<tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_wscrdVal species=\"ACRU\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"ACSA\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"BEAL\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"FAGR\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"TSCA\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"FRAM\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"PIST\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"PRSE\">0</tr_wscrdVal>");
    oOut.write("<tr_wscrdVal species=\"QURU\">0</tr_wscrdVal>");
    oOut.write("</tr_whatSaplingCrownRadDiam>");
    oOut.write("<tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_wachhVal species=\"ACRU\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"ACSA\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"BEAL\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"FAGR\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"TSCA\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"FRAM\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"PIST\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"PRSE\">0</tr_wachhVal>");
    oOut.write("<tr_wachhVal species=\"QURU\">0</tr_wachhVal>");
    oOut.write("</tr_whatAdultCrownHeightHeight>");
    oOut.write("<tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_wschhVal species=\"ACRU\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"ACSA\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"BEAL\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"FAGR\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"TSCA\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"FRAM\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"PIST\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"PRSE\">0</tr_wschhVal>");
    oOut.write("<tr_wschhVal species=\"QURU\">0</tr_wschhVal>");
    oOut.write("</tr_whatSaplingCrownHeightHeight>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"ACRU\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"ACSA\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"BEAL\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"FAGR\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"TSCA\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"FRAM\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"PIST\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"PRSE\">30.0</tr_chVal>");
    oOut.write("<tr_chVal species=\"QURU\">30.0</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"ACRU\">0.108</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"ACSA\">0.107</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"BEAL\">0.109</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"FAGR\">0.152</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"TSCA\">0.1</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"FRAM\">0.095</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"PIST\">0.087</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"PRSE\">0.116</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"QURU\">0.119</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"ACRU\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"ACSA\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"BEAL\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"FAGR\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"TSCA\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"FRAM\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"PIST\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"PRSE\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"QURU\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"ACRU\">0.49</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"ACSA\">0.58</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"BEAL\">0.54</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"FAGR\">0.664</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"TSCA\">0.846</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"FRAM\">0.319</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"PIST\">0.413</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"PRSE\">0.37</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"QURU\">0.413</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"ACRU\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"ACSA\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"BEAL\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"FAGR\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"TSCA\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"FRAM\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"PIST\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"PRSE\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"QURU\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"ACRU\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"ACSA\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"BEAL\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"FAGR\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"TSCA\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"FRAM\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"PIST\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"PRSE\">0.75</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"QURU\">0.75</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_interceptDiam10ToDBH>");
    oOut.write("<tr_idtdVal species=\"ACRU\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"ACSA\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"BEAL\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"FAGR\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"TSCA\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"FRAM\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"PIST\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"PRSE\">0.0</tr_idtdVal>");
    oOut.write("<tr_idtdVal species=\"QURU\">0.0</tr_idtdVal>");
    oOut.write("</tr_interceptDiam10ToDBH>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"ACRU\">0.063</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"ACSA\">0.062333334</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"BEAL\">0.063</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"FAGR\">0.03533333</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"TSCA\">0.024333334</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"FRAM\">0.055999998</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"PIST\">0.033333335</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"PRSE\">0.045</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"QURU\">0.042</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"ACRU\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"ACSA\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"BEAL\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"FAGR\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"TSCA\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"FRAM\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"PIST\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"PRSE\">0.03</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"QURU\">0.03</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("</allometry>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.779</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"ACRU\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"ACSA\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"BEAL\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"FAGR\">0.064</li_lecVal>");
    oOut.write("<li_lecVal species=\"TSCA\">0.064</li_lecVal>");
    oOut.write("<li_lecVal species=\"FRAM\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"PIST\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"PRSE\">0.399</li_lecVal>");
    oOut.write("<li_lecVal species=\"QURU\">0.566</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snagAgeClass1>0</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>0</li_snagAgeClass2>");
    oOut.write("</gliLight>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_minSunAngle>0.779</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_quadratLightHeight>0.675</li_quadratLightHeight>");
    oOut.write("</quadratGliLight>");
    oOut.write("<gliMap>");
    oOut.write("<li_minSunAngle>0.779</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_mapLightHeight>0.0</li_mapLightHeight>");
    oOut.write("</gliMap>");
    oOut.write("<growth>");
    oOut.write("<gr_adultConstAreaInc>");
    oOut.write("<gr_acaiVal species=\"ACRU\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"ACSA\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"BEAL\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"FAGR\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"TSCA\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"FRAM\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"PIST\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"PRSE\">0.3</gr_acaiVal>");
    oOut.write("<gr_acaiVal species=\"QURU\">0.3</gr_acaiVal>");
    oOut.write("</gr_adultConstAreaInc>");
    oOut.write("<gr_adultConstRadialInc>");
    oOut.write("<gr_acriVal species=\"ACRU\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"ACSA\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"BEAL\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"FAGR\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"TSCA\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"FRAM\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"PIST\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"PRSE\">3</gr_acriVal>");
    oOut.write("<gr_acriVal species=\"QURU\">3</gr_acriVal>");
    oOut.write("</gr_adultConstRadialInc>");
    oOut.write("<gr_asympDiameterGrowth>");
    oOut.write("<gr_adgVal species=\"ACRU\">0.167</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"ACSA\">0.125</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"BEAL\">0.169</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"FAGR\">0.152</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"TSCA\">0.229</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"FRAM\">0.226</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"PIST\">0.23</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"PRSE\">0.249</gr_adgVal>");
    oOut.write("<gr_adgVal species=\"QURU\">0.266</gr_adgVal>");
    oOut.write("</gr_asympDiameterGrowth>");
    oOut.write("<gr_slopeGrowthResponse>");
    oOut.write("<gr_sgrVal species=\"ACRU\">0.027</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"ACSA\">0.159</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"BEAL\">0.137</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"FAGR\">0.075</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"TSCA\">0.051</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"FRAM\">0.025</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"PIST\">0.019</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"PRSE\">0.064</gr_sgrVal>");
    oOut.write("<gr_sgrVal species=\"QURU\">0.022</gr_sgrVal>");
    oOut.write("</gr_slopeGrowthResponse>");
    oOut.write("<gr_relGrowthDiamExp>");
    oOut.write("<gr_rgdeVal species=\"ACRU\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"ACSA\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"BEAL\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"FAGR\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"TSCA\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"FRAM\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"PIST\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"PRSE\">1.0</gr_rgdeVal>");
    oOut.write("<gr_rgdeVal species=\"QURU\">1.0</gr_rgdeVal>");
    oOut.write("</gr_relGrowthDiamExp>");
    oOut.write("</growth>");
    oOut.write("</paramFile>");
    
    oOut.close();
    return sFileName;
  }
}
